# -*-coding: utf-8
"""
    flaskext.rbac
    ~~~~~~~~~~~~~

    Adds Role-based Access Control modules to application
"""

import itertools
from collections import defaultdict

from flask import request, abort, _request_ctx_stack

try:
    from flask import _app_ctx_stack
except ImportError:
    _app_ctx_stack = None

try:
    from flask_login import (current_user,
                             AnonymousUserMixin as anonymous_model)
except ImportError:
    current_user, anonymous_model = None, None

from .model import RoleMixin, UserMixin, anonymous


__all__ = ['RBAC', 'RoleMixin', 'UserMixin']


connection_stack = _app_ctx_stack or _request_ctx_stack


class AccessControlList(object):
    """
    This class record rules for access controling.
    """

    def __init__(self):
        self._allowed = []
        self._denied = []
        self._exempt = []
        self.seted = False

    def allow(self, role, method, resource, with_children=True):
        """Add allowing rules.

        :param role: Role of this rule.
        :param method: Method to allow in rule, include GET, POST, PUT etc.
        :param resource: Resource also view function.
        :param with_children: Allow role's children in rule as well
                              if with_children is `True`
        """
        if with_children:
            for r in role.get_children():
                permission = (r.get_name(), method, resource)
                if permission not in self._allowed:
                    self._allowed.append(permission)
        if role == 'anonymous':
            permission = (role, method, resource)
        else:
            permission = (role.get_name(), method, resource)
        if permission not in self._allowed:
            self._allowed.append(permission)

    def deny(self, role, method, resource, with_children=False):
        """Add denying rules.

        :param role: Role of this rule.
        :param method: Method to deny in rule, include GET, POST, PUT etc.
        :param resource: Resource also view function.
        :param with_children: Deny role's children in rule as well
                              if with_children is `True`
        """
        if with_children:
            for r in role.get_children():
                permission = (r.get_name(), method, resource)
                if permission not in self._denied:
                    self._denied.append(permission)
        permission = (role.get_name(), method, resource)
        if permission not in self._denied:
            self._denied.append(permission)

    def exempt(self, resource):
        """Exempt a view function from being checked permission

        :param resource: The view function exempt from checking.
        """
        if resource not in self._exempt:
            self._exempt.append(resource)

    def is_allowed(self, role, method, resource):
        """Check whether role is allowed to access resource

        :param role: Role to be checked.
        :param method: Method to be checked.
        :param resource: View function to be checked.
        """
        return (role, method, resource) in self._allowed

    def is_denied(self, role, method, resource):
        """Check wherther role is denied to access resource

        :param role: Role to be checked.
        :param method: Method to be checked.
        :param resource: View function to be checked.
        """
        return (role, method, resource) in self._denied

    def is_exempt(self, resource):
        """Return whether resource is exempted.

        :param resource: View function to be checked.
        """
        return resource in self._exempt


class _RBACState(object):
    '''Records configuration for Flask-RBAC'''
    def __init__(self, rbac, app):
        self.rbac = rbac
        self.app = app


class RBAC(object):
    """This class implements role-based access control module in Flask.
    There are two way to initialize Flask-RBAC::

        app = Flask(__name__)
        rbac = RBAC(app)

    or::

        rbac = RBAC
        def create_app():
            app = Flask(__name__)
            rbac.init_app(app)
            return app

    :param app: the Flask object
    :param role_model: custom role model
    :param user_model: custom user model
    :param user_loader: custom user loader, used to load current user
    :param permission_failed_hook: called when permission denied.
    """

    def __init__(self, app=None, **kwargs):
        """Initialize with app."""
        self.acl = AccessControlList()
        self.before_acl = {'allow': [], 'deny': []}

        self._role_model = kwargs.get('role_model', RoleMixin)
        self._user_model = kwargs.get('user_model', UserMixin)
        self._user_loader = kwargs.get('user_loader', lambda: current_user)
        self.permission_failed_hook = kwargs.get('permission_failed_hook')

        if app is not None:
            self.app = app
            self.init_app(app)
        else:
            self.app = None

    def init_app(self, app):
        """Initialize application in Flask-RBAC.
        Adds (RBAC, app) to flask extensions.
        Adds hook to authenticate permission before request.

        :param app: Flask object
        """

        app.config.setdefault('RBAC_USE_WHITE', False)
        self.use_white = app.config['RBAC_USE_WHITE']

        if not hasattr(app, 'extensions'):
            app.extensions = {}
        app.extensions['rbac'] = _RBACState(self, app)

        self.acl.allow(anonymous, 'GET', 'static')
        app.before_first_request(self._setup_acl)

        app.before_request(self._authenticate)

    def as_role_model(self, model_cls):
        """A decorator to set custom model or role.

        :param model_cls: Model of role.
        """
        self._role_model = model_cls
        return model_cls

    def as_user_model(self, model_cls):
        """A decorator to set custom model or user.

        :param model_cls: Model of user.
        """
        self._user_model = model_cls
        return model_cls

    def set_role_model(self, model):
        """Set custom model of role.

        :param model: Model of role.
        """
        self._role_model = model

    def set_user_model(self, model):
        """Set custom model of User

        :param model: Model of user
        """
        self._user_model = model

    def set_user_loader(self, loader):
        """Set user loader, which is used to load current user.
        An example::

            from flask_login import current_user
            rbac.set_user_loader(lambda: current_user)

        :param loader: Current user function.
        """
        self._user_loader = loader

    def set_hook(self, hook):
        """Set hook which called when permission is denied
        If you haven't set any hook, Flask-RBAC will call::

            abort(403)

        :param hook: Hook function
        """
        self.permission_failed_hook = hook

    def has_permission(self, method, endpoint, user=None):
        """Return does the current user can access the resource.
        Example::

            @app.route('/some_url', methods=['GET', 'POST'])
            @rbac.allow(['anonymous'], ['GET'])
            def a_view_func():
                return Response('Blah Blah...')

        If you are not logged.

        `rbac.has_permission('GET', 'a_view_func')` return True.
        `rbac.has_permission('POST', 'a_view_func')` return False.

        :param method: The method wait to check.
        :param endpoint: The application endpoint.
        :param user: user who you need to check. Current user by default.
        """
        app = self.get_app()
        _user = user or self._user_loader()
        if not hasattr(_user, 'get_roles'):
            roles = [anonymous]
        else:
            roles = _user.get_roles()
        return self._check_permission(roles, method, endpoint)

    def allow(self, roles, methods, with_children=True, endpoint=None):
        """This is a decorator function.

        You can allow roles to access the view func with it.

        An example::

            @app.route('/website/setting', methods=['GET', 'POST'])
            @rbac.allow(['administrator', 'super_user'], ['GET', 'POST'])
            def website_setting():
                return Response('Setting page.')

        :param roles: List, each name of roles. Please note that,
                      `anonymous` is refered to anonymous.
                      If you add `anonymous` to the rule,
                      everyone can access the resource,
                      unless you deny other roles.
        :param methods: List, each name of methods.
                        methods is valid in ['GET', 'POST', 'PUT', 'DELETE']
        :param with_children: Whether allow children of roles as well.
                              True by default.
        """
        def decorator(view_func):
            _methods = [m.upper() for m in methods]
            resource = [endpoint or view_func.__name__]
            for r, m, v in itertools.product(roles, _methods, resource):
                self.before_acl['allow'].append((r, m, v, with_children))
            return view_func
        return decorator

    def deny(self, roles, methods, with_children=False, endpoint=None):
        """This is a decorator function.

        You can deny roles to access the view func with it.

        An example::

            @app.route('/article/post', methods=['GET', 'POST'])
            @rbac.deny(['anonymous', 'unactivated_role'], ['GET', 'POST'])
            def article_post():
                return Response('post page.')

        :param roles: List, each name of roles.
        :param methods: List, each name of methods.
                        methods is valid in ['GET', 'POST', 'PUT', 'DELETE']
        :param with_children: Whether allow children of roles as well.
                              True by default.
        """
        def decorator(view_func):
            _methods = [m.upper() for m in methods]
            resource = [endpoint or view_func.__name__]
            for r, m, v in itertools.product(roles, _methods, resource):
                self.before_acl['deny'].append((r, m, v, with_children))
            return view_func
        return decorator

    def exempt(self, view_func):
        """Exempt a view function from being checked permission.
        It is useful when you are using white list checking.

        Example::

            @app.route('/everyone/can/access')
            @rbac.exempt
            def everyone_can_access():
                return 'Hello~'

        :param view_func: The view function going to be exempted.
        """
        self.acl.exempt(view_func.__name__)
        return view_func

    def get_app(self, reference_app=None):
        """Helper method that implements the logic to look up an application.
        """
        if reference_app is not None:
            return reference_app
        if self.app is not None:
            return self.app
        ctx = connection_stack.top
        if ctx is not None:
            return ctx.app
        raise RuntimeError('application not registered on rbac '
                           'instance and no application bound '
                           'to current context')

    def _authenticate(self):
        app = self.get_app()
        assert app, "Please initialize your application into Flask-RBAC."
        assert self._role_model, "Please set role model before authenticate."
        assert self._user_model, "Please set user model before authenticate."
        assert self._user_loader, "Please set user loader before authenticate."

        current_user = self._user_loader()

        # Compatible with flask-login anonymous user
        if hasattr(current_user, '_get_current_object'):
            current_user = current_user._get_current_object()

        if (current_user is not None
                and not isinstance(current_user,
                                   (self._user_model, anonymous_model))):
            raise TypeError(
                "%s is not an instance of %s" %
                (current_user, self._user_model.__class__))

        resource = request.endpoint
        if not resource:
            abort(404)

        method = request.method

        if not hasattr(current_user, 'get_roles'):
            roles = [anonymous]
        else:
            roles = current_user.get_roles()

        permit = self._check_permission(roles, method, resource)

        if not permit:
            return self._deny_hook()

    def _check_permission(self, roles, method, resource):

        if self.acl.is_exempt(resource):
            return True

        _roles = set()
        _methods = set(['*', method])
        _resources = set([None, resource])

        if self.use_white:
            _roles.add(anonymous)

        is_allowed = None
        _roles.update(roles)

        if not self.acl.seted:
            self._setup_acl()

        for r, m, res in itertools.product(_roles, _methods, _resources):
            if self.acl.is_denied(r.get_name(), m, res):
                return False

            if not is_allowed and self.acl.is_allowed(r.get_name(), m, res):
                is_allowed = True
                break

        if self.use_white:
            permit = (is_allowed is True)
        else:
            permit = (is_allowed is not False)

        return permit

    def _deny_hook(self):
        if self.permission_failed_hook:
            return self.permission_failed_hook()
        else:
            abort(403)

    def _setup_acl(self):
        for rn, method, resource, with_children in self.before_acl['allow']:
            role = self._role_model.get_by_name(rn)
            if rn == 'anonymous':
                role = anonymous
            else:
                role = self._role_model.get_by_name(rn)
            self.acl.allow(role, method, resource, with_children)

        if not self.use_white:
            to_deny_map = defaultdict(list)
            all_roles = {x.get_name() if not isinstance(x, str)
                    else x for x in self._role_model.get_all()}

            for role, method, resource, with_children in self.before_acl['allow']:
                to_deny_map[(resource, role, with_children)].append(method)
            for k, methods in to_deny_map.items():
                view, role, with_children, = k
                for r, m in itertools.product(all_roles - {role}, methods):
                    rule = (r, m, view, with_children)
                    if rule not in self.before_acl['allow']:
                        self.before_acl['deny'].append(rule)

        for rn, method, resource, with_children in self.before_acl['deny']:
            role = self._role_model.get_by_name(rn)
            self.acl.deny(role, method, resource, with_children)
        self.acl.seted = True



class RoleMixin(object):
    """This provides implementations for the methods that Flask-RBAC wants
    the role model to have.

    :param name: The name of role.
    """

    roles = {}

    def __init__(self, name=None):
        self.name = name
        if not hasattr(self.__class__, 'parents'):
            self.parents = set()
        if not hasattr(self.__class__, 'children'):
            self.children = set()
        RoleMixin.roles[name] = self

    def get_name(self):
        """Return the name of this role"""
        return self.name

    def add_parent(self, parent):
        """Add a parent to this role,
        and add role itself to the parent's children set.
        you should override this function if neccessary.

        Example::

            logged_user = RoleMixin('logged_user')
            student = RoleMixin('student')
            student.add_parent(logged_user)

        :param parent: Parent role to add in.
        """
        parent.children.add(self)
        self.parents.add(parent)

    def add_parents(self, *parents):
        """Add parents to this role. Also should override if neccessary.
        Example::

            editor_of_articles = RoleMixin('editor_of_articles')
            editor_of_photonews = RoleMixin('editor_of_photonews')
            editor_of_all = RoleMixin('editor_of_all')
            editor_of_all.add_parents(editor_of_articles, editor_of_photonews)

        :param parents: Parents to add.
        """
        for parent in parents:
            self.add_parent(parent)

    def get_parents(self):
        for parent in self.parents:
            yield parent
            for grandparent in parent.get_parents():
                yield grandparent

    def get_children(self):
        for child in self.children:
            yield child
            for grandchild in child.get_children():
                yield grandchild

    @staticmethod
    def get_by_name(name):
        """A static method to return the role which has the input name.

        :param name: The name of role.
        """
        return RoleMixin.roles[name]

    @classmethod
    def get_all(cls):
        """Return all existing roles
        """
        return cls.roles


class UserMixin(object):
    """This provides implementations for the methods that Flask-RBAC wants
    the user model to have.

    :param roles: The roles of this user should have.
    """

    def __init__(self, roles=[]):
        self.roles = set(roles)

    def add_role(self, role):
        """Add a role to this user.

        :param role: Role to add.
        """
        self.roles.add(role)

    def add_roles(self, *roles):
        """Add roles to this user.

        :param roles: Roles to add.
        """
        for role in roles:
            self.add_role(role)

    def get_roles(self):
        for role in self.roles:
            yield role


anonymous = RoleMixin('anonymous')



import unittest

from flask import Flask, Response, make_response
from flask_login import current_user as login_user

from flask_rbac import RBAC, UserMixin, RoleMixin

import functools


class Role(RoleMixin):
    def __repr__(self):
        return '<Role %s>' % self.name

class User(UserMixin):
    def __repr__(self):
        return '<User %s>' % self.roles

everyone = Role('everyone')
logged_role = Role('logged_role')
staff_role = Role('staff_role')
other_role = Role('other_role')
special = Role('special')

logged_role.add_parent(everyone)
staff_role.add_parents(everyone, logged_role)

anonymous = User(roles=[everyone])
normal_user = User(roles=[logged_role])
staff_role_user = User(roles=[staff_role])
special_user = User(roles=[special])
many_roles_user = User(roles=[logged_role, other_role, everyone])

current_user = anonymous

def rewrite_decorator(viewfunc):
    @functools.wraps(viewfunc)
    def newfunc(*args, **kwargs):
        return viewfunc(*args, **kwargs)
    return newfunc

def makeapp(with_factory, use_white, before_decorator, after_decorator):
    global current_user
    app = Flask(__name__)
    app.debug = True

    if use_white:
        app.config['RBAC_USE_WHITE'] = True
    else:
        app.config['RBAC_USE_WHITE'] = False

    if with_factory:
        rbac = RBAC()
        rbac.init_app(app)
    else:
        rbac = RBAC(app)

    rbac.set_user_loader(lambda: current_user)
    rbac.set_user_model(User)
    rbac.set_role_model(Role)

    @app.route('/')
    @after_decorator
    @rbac.allow(roles=['everyone'], methods=['GET'])
    @before_decorator
    def index():
        return Response('index')

    @app.route('/a')
    @after_decorator
    @rbac.allow(roles=['special'], methods=['GET'])
    @before_decorator
    def a():
        return Response('Hello')

    @app.route('/b', methods=['GET', 'POST'])
    @after_decorator
    @rbac.allow(roles=['logged_role'], methods=['GET'])
    @rbac.allow(roles=['staff_role', 'special'], methods=['POST'])
    @before_decorator
    def b():
        return Response('Hello from /b')

    @app.route('/c')
    @after_decorator
    @rbac.allow(roles=['everyone'], methods=['GET'])
    @rbac.deny(roles=['logged_role'], methods=['GET'], with_children=False)
    @rbac.allow(roles=['staff_role'], methods=['GET'])
    @before_decorator
    def c():
        return Response('Hello from /c')

    @app.route('/d')
    @after_decorator
    @rbac.deny(roles=['everyone'], methods=['GET'])
    @before_decorator
    def d():
        return Response('Hello from /d')

    @app.route('/e')
    @after_decorator
    @rbac.deny(roles=['everyone'], methods=['GET'], with_children=True)
    @before_decorator
    def e():
        return Response('Hello from /e')

    @app.route('/f', methods=['POST'])
    @after_decorator
    @rbac.deny(roles=['logged_role'], methods=['POST'])
    @before_decorator
    def f():
        return Response('Hello from /f')

    @app.route('/g', methods=['GET'])
    @after_decorator
    @rbac.exempt
    @before_decorator
    def g():
        return Response('Hello from /g')

    @app.route('/h', methods=['GET'])
    @after_decorator
    @rbac.allow(['anonymous'], methods=['GET'], with_children=False)
    @before_decorator
    def h():
        return Response('Hello from /h')

    return app


class UseWhiteApplicationUnitTests(unittest.TestCase):

    def setUp(self):
        self.app = makeapp(with_factory=False, use_white=True, before_decorator=rewrite_decorator, after_decorator=rewrite_decorator)
        self.client = self.app.test_client()
        self.rbac = self.app.extensions['rbac'].rbac

    def test_set_user_loader(self):
        global current_user
        self.assertEqual(self.rbac._user_loader(), current_user)
        self.rbac.set_user_loader(lambda: staff_role_user)
        self.assertEqual(self.rbac._user_loader(), staff_role_user)
        # Restore
        self.rbac.set_user_loader(lambda: current_user)

    def test_allow_get_view(self):
        global current_user
        current_user = anonymous
        self.assertEqual(self.client.open('/').data.decode('utf-8'), 'index')

        current_user = normal_user
        self.assertEqual(self.client.open('/').data.decode('utf-8'), 'index')
        self.assertEqual(self.client.open('/b').data.decode('utf-8'), 'Hello from /b')

        current_user = staff_role_user
        self.assertEqual(self.client.open('/').data.decode('utf-8'), 'index')
        self.assertEqual(self.client.open('/b').data.decode('utf-8'), 'Hello from /b')

        current_user = special_user
        self.assertEqual(self.client.open('/a').data.decode('utf-8'), 'Hello')

    def test_deny_get_view(self):
        global current_user
        current_user = special_user
        self.assertEqual(self.client.open('/').status_code, 403)
        self.assertEqual(self.client.open('/b').status_code, 403)

        current_user = anonymous
        self.assertEqual(self.client.open('/a').status_code, 403)

        current_user = normal_user
        self.assertEqual(self.client.open('/a').status_code, 403)

        current_user = staff_role_user
        self.assertEqual(self.client.open('/a').status_code, 403)

    def test_allow_post_view(self):
        global current_user
        current_user = staff_role_user
        self.assertEqual(self.client.post('/b').data.decode('utf-8'), 'Hello from /b')

        current_user = special_user
        self.assertEqual(self.client.post('/b').data.decode('utf-8'), 'Hello from /b')

    def test_deny_post_view(self):
        global current_user
        current_user = anonymous
        self.assertEqual(self.client.post('/b').status_code, 403)

        current_user = normal_user
        self.assertEqual(self.client.post('/b').status_code, 403)

    def test_complicate_get_view(self):
        global current_user
        current_user = anonymous
        self.assertEqual(self.client.open('/c').data.decode('utf-8'), 'Hello from /c')

        current_user = normal_user
        self.assertEqual(self.client.open('/c').status_code, 403)

        current_user = staff_role_user
        self.assertEqual(self.client.open('/c').data.decode('utf-8'), 'Hello from /c')

    def test_hook(self):
        global current_user
        current_user = special_user
        self.rbac.set_hook(lambda: make_response('Permission Denied', 403))
        self.assertEqual(self.client.open('/').status_code, 403)
        self.assertEqual(self.client.open('/').data.decode('utf-8'), 'Permission Denied')

    def test_has_permission(self):
        global current_user

        current_user = anonymous
        self.assertTrue(self.rbac.has_permission('GET', 'index'))
        self.assertTrue(self.rbac.has_permission('GET', 'c'))
        self.assertFalse(self.rbac.has_permission('GET', 'a'))
        self.assertFalse(self.rbac.has_permission('POST', 'index'))

        current_user = special_user
        self.assertTrue(self.rbac.has_permission('GET', 'a'))
        self.assertTrue(self.rbac.has_permission('POST', 'b'))
        self.assertFalse(self.rbac.has_permission('GET', 'c'))

        current_user = anonymous
        self.assertTrue(self.rbac.has_permission('POST', 'b', special_user))

        current_user = None
        self.assertTrue(self.rbac.has_permission('GET', 'h'))
        self.assertEqual(self.client.open('/h').data.decode('utf-8'), 'Hello from /h')


    def test_exempt(self):
        global current_user

        current_user = anonymous
        self.assertEqual(self.client.open('/g').data.decode('utf-8'), 'Hello from /g')

        current_user = special_user
        self.assertEqual(self.client.open('/g').data.decode('utf-8'), 'Hello from /g')

        current_user = normal_user
        self.assertEqual(self.client.open('/g').data.decode('utf-8'), 'Hello from /g')


class NoWhiteApplicationUnitTests(unittest.TestCase):

    def setUp(self):
        self.app = makeapp(with_factory=False, use_white=False, before_decorator=rewrite_decorator, after_decorator=rewrite_decorator)
        self.client = self.app.test_client()
        self.rbac = self.app.extensions['rbac'].rbac

    def test_allow_get_view(self):
        global current_user
        current_user = normal_user
        self.assertEqual(self.client.open('/d').data.decode('utf-8'), 'Hello from /d')

        current_user = staff_role_user
        self.assertEqual(self.client.open('/d').data.decode('utf-8'), 'Hello from /d')

    def test_deny_get_view(self):
        global current_user
        current_user = anonymous
        self.assertEqual(self.client.open('/d').status_code, 403)
        self.assertEqual(self.client.open('/e').status_code, 403)

        current_user = normal_user
        self.assertEqual(self.client.open('/e').status_code, 403)

        current_user = staff_role_user
        self.assertEqual(self.client.open('/e').status_code, 403)

    def test_allow_post_view(self):
        global current_user
        current_user = anonymous
        self.assertEqual(self.client.post('/f').data.decode('utf-8'), 'Hello from /f')

        current_user = staff_role_user
        self.assertEqual(self.client.post('/f').data.decode('utf-8'), 'Hello from /f')

    def test_deny_post_view(self):
        global current_user
        current_user = normal_user
        self.assertEqual(self.client.post('/f').status_code, 403)

    def test_has_permission(self):
        global current_user
        current_user = normal_user
        self.assertTrue(self.rbac.has_permission('GET', 'd'))
        self.assertFalse(self.rbac.has_permission('POST', 'f'))


class RoleMixInUnitTests(unittest.TestCase):

    def test_role_get_name(self):
        self.assertEqual(everyone.get_name(), 'everyone')
        self.assertEqual(logged_role.get_name(), 'logged_role')
        self.assertEqual(staff_role.get_name(), 'staff_role')

    def test_add_parent(self):
        normal_role = Role('normal')
        base_role = Role('base')
        normal_role.add_parent(base_role)

        self.assertIn(base_role, normal_role.parents)

    def test_add_parents(self):
        normal_role = Role('normal')
        parent_role = Role('parent')
        another_parent_role = Role('another_parent')
        normal_role.add_parents(parent_role, another_parent_role)

        self.assertIn(parent_role, normal_role.parents)
        self.assertIn(another_parent_role, normal_role.parents)

    def test_get_parents(self):
        everyone_parents = set()
        logged_role_parents = set()
        staff_role_parents = set()

        for p in everyone.get_parents():
            everyone_parents.add(p)
        for p in logged_role.get_parents():
            logged_role_parents.add(p)
        for p in staff_role.get_parents():
            staff_role_parents.add(p)

        self.assertEqual(everyone_parents, set([]))
        self.assertEqual(logged_role_parents, set([everyone]))
        self.assertEqual(staff_role_parents, set([everyone, logged_role]))

    def test_get_children(self):
        everyone_children = set()
        logged_role_children = set()
        staff_role_children = set()

        for c in everyone.get_children():
            everyone_children.add(c)
        for c in logged_role.get_children():
            logged_role_children.add(c)
        for c in staff_role.get_children():
            staff_role_children.add(c)

        self.assertEqual(everyone_children, set([logged_role, staff_role]))
        self.assertEqual(logged_role_children, set([staff_role]))
        self.assertEqual(staff_role_children, set([]))


class UserMixInUnitTests(unittest.TestCase):

    def test_add_role(self):
        new_role = Role('new_for_test')
        another_role = Role('for_test')
        the_third_role = Role('The_third_man')

        user_one = User()
        user_two = User()
        user_three = User()

        user_one.add_role(another_role)
        user_three.add_role(new_role)
        user_three.add_role(another_role)

        self.assertEqual(user_one.roles, set([another_role]))
        self.assertEqual(user_two.roles, set([]))
        self.assertEqual(user_three.roles, set([new_role, another_role]))

    def test_get_roles(self):
        normal_user_roles = set()
        staff_role_user_roles = set()
        many_roles_user_roles = set()

        for r in normal_user.get_roles():
            normal_user_roles.add(r)
        for r in staff_role_user.get_roles():
            staff_role_user_roles.add(r)
        for r in many_roles_user.get_roles():
            many_roles_user_roles.add(r)

        self.assertEqual(normal_user_roles, set([logged_role]))
        self.assertEqual(staff_role_user_roles, set([staff_role]))
        self.assertEqual(many_roles_user_roles, set([everyone, logged_role, other_role]))


class DecoratorUnitTests(unittest.TestCase):

    def setUp(self):
        self.rbac = RBAC()

        @self.rbac.as_role_model
        class RoleModel(RoleMixin):
            pass

        @self.rbac.as_user_model
        class UserModel(UserMixin):
            pass

        self.rm = RoleModel
        self.um = UserModel

    def test_as_role_model(self):
        self.assertTrue(self.rbac._role_model is self.rm)

    def test_as_user_model(self):
        self.assertTrue(self.rbac._user_model is self.um)


class DefaultUserLoaderUnitTests(unittest.TestCase):

    def setUp(self):
        self.rbac = RBAC()

    def test_default_user_loader(self):
        self.assertEqual(self.rbac._user_loader(), login_user)



from setuptools import setup, find_packages
from os.path import dirname, realpath, join

CURRENT_DIR = dirname(realpath(__file__))

with open(join(CURRENT_DIR, "README.rst")) as long_description_file:
    long_description = long_description_file.read()


setup(
    name="Flask-RBAC",
    version="0.5.0",
    url="https://github.com/shonenada/flask-rbac",
    author="Yaoda Liu",
    author_email="shonenada@gmail.com",
    description="RBAC support for Flask",
    long_description=long_description,
    zip_safe=False,
    packages=find_packages(exclude=["docs"]),
    include_package_data=True,
    platforms="any",
    install_requires=["Flask>=0.10"],
    classifiers=[
        "Framework :: Flask",
        "Environment :: Web Environment",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ]
)
